package gs.test.v0.definition

import gs.test.v0.definition.pos.SourcePosition
import scala.reflect.*

sealed abstract class Assertion(val name: String)

object Assertion:

  private def success(): Either[TestFailure, Unit] = Right(())

  // TODO: Use a smart rendering solution, consider diffs.
  // For now, this is fine -- add a diff library later as data.
  def renderInput[A](value: A): String = value.toString()

  case object IsEqualTo extends Assertion("isEqualTo"):

    def evaluate[A: ClassTag](
      candidate: A,
      expected: A
    )(
      using
      CanEqual[A, A]
    )(
      using
      pos: SourcePosition
    ): Either[TestFailure, Unit] =
      if candidate == expected then success()
      else
        val runtimeType = classTag[A].runtimeClass.getName()
        Left(
          TestFailure.AssertionFailed(
            assertionName = name,
            inputs = Map(
              "candidate" -> runtimeType,
              "expected"  -> runtimeType
            ),
            message =
              s"'${renderInput(candidate)}' was not equal to '${renderInput(candidate)}'",
            pos = pos
          )
        )

  case object IsTrue extends Assertion("isTrue"):

    def evaluate(
      candidate: Boolean
    )(
      using
      pos: SourcePosition
    ): Either[TestFailure, Unit] =
      if candidate then success()
      else
        Left(
          TestFailure.AssertionFailed(
            assertionName = name,
            inputs = Map("candidate" -> "Boolean"),
            message = s"Expected '$candidate' to be 'true'.",
            pos = pos
          )
        )

end Assertion
